#
# Licensed Materials - Property of IBM
# Rational ClearQuest 
# (C) Copyright IBM Corp. 2011, 2014 All Rights Reserved
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#

use strict;

sub AddSiteArg;
sub AddSendToArg;

sub Usage {
    print <<EOF;

This script is used initially to extract ClearQuest records for Full-Text 
Search and to generate encrypted password and key file. It is also used 
to optimize the Full-Text Search.

Usage: cqperl $0 [-jvmpar <JVM option>] {-batch | -update | -repair <repair_file> | -optimize} <cq_search_properties_file> [-site <site_name>]
or
Usage: cqperl $0 -encrypt -s <dbset> -d <dbname> -u <user> -p <password> -kf <output_key_file_name> -pf <output_encrypted_password_file_name>

Parameters:
  -jvmpar        Set the JVM option
  -batch         Runs the Record Extractor in Batch mode
  -update        Runs the Record Extractor in Update mode
  -repair <file> Runs the Record Extractor in Repair mode
  -optimize      Optimizes the search engine index to reduce fragmentation
  -sendto        To which index to send the command: [both | normal | typeahead]
  -site <name>   The site name (when MultiSite'ed)
  -encrypt       Runs the encryption command
  -s <dbset>     The database set name
  -d <dbname>    The user database name
  -u <user>      The user name
  -p <password>  THe password
  -kf <file>     The file to which the password encryption key is to be written to
  -pf <file>     The file to which the encrypted password is to be written to
  
Example 1. Extract existing records for indexing:
cqperl $0 -batch ExtractionConfig.xml -site cqserver

Example 2. Extract existing records for indexing with JVM option:
cqperl $0 -jvmpar "-Xms512m -Xmx1024m" -batch ExtractionConfig.xml -site cqserver

Example 3. Extract records that were not processed during batch / update
extraction:
cqperl $0 -repair lostrecords ExtractionConfig.xml

Example 4. Extract new and modified records for indexing (Note: don't
use if it is also running under eWAS)
cqperl $0 -update ExtractionConfig.xml

Example 5. Generate an encrypted password and key file:
cqperl $0 -encrypt -u admin -p "" -s 7.0.0 -d SAMPL -kf key.txt -pf pwd.txt

Example 6. Optimize the search engine index
cqperl $0 -optimize CQ-SAMPL-TextSearch.xml
EOF
    exit 1;
}

# See if they requested usage info...
if (($#ARGV < 1) || ($ARGV[0] =~ /^-[h\?]/i)) {
    Usage();
}

my $classpath;
my $java;

if ($ENV{'OS'} =~ /Windows/i) {
    $classpath = './cqtsdbcrawler.jar;./cqjni.jar';
    $java = $ENV{'JRE_HOME'}.'\bin\java.exe';
}else {
    $classpath = './cqtsdbcrawler.jar:./cqjni.jar';
    $java = $ENV{'CQ_HOME'}.'/../common/java/jre/bin/java';
}

my $cmd = "\"$java\" -version";
my $cmdout = `$cmd 2>&1`;

# Check return code
if ($? != 0) {
    die "Can not find executable Java\nPlease check if ClearQuest is installed correctly.\n";
}

# Verify java version
my $version5 = 0;
if ($cmdout =~ /^java version "(\d+)\.(\d+)\./) {
    if ($1 > 1) {
        $version5 = 1;
    }
    elsif (($1 == 1) && ($2 >= 5)) {
        $version5 = 1;
    }
}

if ($version5 == 0) {
    die "This utility needs Java version 1.5 or greater.\n";
}

if ($ARGV[0] eq "-encrypt") {
    if (($#ARGV != 12)) {
        Usage();
    }
    my @para;
    for (my $index = 0; $index < 12; $index++){
        $para[$index] = "\"$ARGV[$index+1]\"";
    }
    my $encrypt = "\"$java\" -classpath $classpath com.ibm.rational.clearquest.cqtscrawler.common.EncryptPassword @para";
    system($encrypt);
    exit 0;
}
$cmd = "\"$java\" -classpath $classpath com.ibm.rational.clearquest.cqtscrawler.batch.ExtractorCommand $ARGV[0] \"$ARGV[1]\"";

if ($ARGV[0] =~ /^-batch|update|optimize/i) {
    unless (-e $ARGV[1]) {
        Exit();
    }

    $cmd .= AddSiteArg(2);
    $cmd .= AddSendToArg(4);
}
elsif (($ARGV[0] =~ /^-repair/i) && ($#ARGV > 1)){
    unless ((-e $ARGV[1]) || (-e $ARGV[2])) {
        Exit();
    }

    $cmd .= " $ARGV[2]";
    $cmd .= AddSiteArg(3);
    $cmd .= AddSendToArg(5);
}
elsif ($ARGV[0] =~ /^-jvmpar/i) {
    $cmd = "\"$java\" $ARGV[1] -classpath $classpath com.ibm.rational.clearquest.cqtscrawler.batch.ExtractorCommand $ARGV[2] \"$ARGV[3]\"";
    if ($ARGV[2] =~ /^-batch|update|optimize/i) {
        unless (-e $ARGV[3]) {
            Exit();
        }

        $cmd .= AddSiteArg(4);
        $cmd .= AddSendToArg(6);
    }
    elsif (($ARGV[2] =~ /^-repair/i) && ($#ARGV > 3)){
        unless ((-e $ARGV[3]) || (-e $ARGV[4])) {
            Exit();
        }

        $cmd .= " $ARGV[4]";
        $cmd .= AddSiteArg(5);
        $cmd .= AddSendToArg(7);
    }
}
else {
    Usage();
}

if (defined($ENV{'CQ_DIAG_BEHAVIOR'})) {
    $ENV{'CQ_DIAG_BEHAVIOR'} = 'HARD_QUERY_LIMIT=0;'.$ENV{'CQ_DIAG_BEHAVIOR'};
}
else {
    $ENV{'CQ_DIAG_BEHAVIOR'} = 'HARD_QUERY_LIMIT=0';
}

print "Executing    : $cmd\n";

system($cmd);

sub Exit {
    die "ERROR : config file $ARGV[1] does NOT exist!\n\n";
}

# If the arg '-site' exists at the specified index, the arg string is returned
sub AddSiteArg
{
    my ($index) = @_;

    my $value = "";

    if (($#ARGV > $index) && ($ARGV[$index] =~ /^-site/i)) {
        $value = " -site \"".$ARGV[$index + 1]."\"";
    }

    return $value;
}

# If the arg '-sendto' exists at the specified index, the arg string is returned
sub AddSendToArg
{
    my ($index) = @_;

    my $value = "";

    if (($#ARGV > $index) && ($ARGV[$index] =~ /^-sendto/i)) {
        $value = " -sendto ".$ARGV[$index + 1];
    }

    return $value;
}
